#include "ISC0901.h"
#include "xil_io.h"
#include "sleep.h"
#include "ram_address.h"
#include <stdint.h>
#include <stdlib.h>

/*
    1. all outputs low
    2. start clk, (test: clk not started until all voltages enabled, seems to be not possible in verilog files)
    3. 500uS delay
    4. enable VCC_CORE (3.3V)
    5. 15ms delay
    6. enable VCC_IO (2.5V)
    7. 4ms delay
    8. enable VCC_BIAS (11.0V)
BIAS voltage at turn on = 11.0V
*/
void init_ISC0901()
{
    int i;

    //Set BIAS values to 0x25 in the first step, 64 levels of bias values available
    for(i=0; i<336*256; i++)
    {
        Xil_Out8(RAM_BIAS_BUF_ADDR+i, 0x25);
    }

    uint8_t cmd_array[20] = {0x00, 0x00, 0x00, 0x00, 0x00,
                             0x00, 0x00, 0x00, 0x00, 0x00,
                             0x00, 0x00, 0x00, 0x00, 0x00,
                             0x00, 0x00, 0x00, 0x00, 0x00};    
    
    //Set CMD to 0x00 in the first step
    for(i=0; i<20; i++)
    {
        Xil_Out8(RAM_CMD_BUF_ADDR+i, cmd_array[i]);
    }    

    Xil_Out32(ISC0901_BIAS_BUF_ADDR_ADDR, RAM_BIAS_BUF_ADDR);   
    Xil_Out32(ISC0901_BIAS_BTT_ADDR, RAM_BIAS_BTT); 
    Xil_Out32(ISC0901_RTEMP_BUF_ADDR_ADDR, RAM_RTEMP_BUF_ADDR);   
    Xil_Out32(ISC0901_RTEMP_BTT_ADDR, RAM_RTEMP_BTT);
    Xil_Out32(ISC0901_CMD_BUF_ADDR_ADDR, RAM_CMD_BUF_ADDR);   
    Xil_Out32(ISC0901_CMD_BTT_ADDR, RAM_CMD_BTT);

    // activate the different voltages
    Xil_Out32(ISC0901_CTRL_REG_ADDR, 0x00); // Reset   

    usleep(500);

    Xil_Out32(ISC0901_CTRL_REG_ADDR, (1<<SENSOR_CORE_PWR_ENA)); // enable VCC_CORE (3.3V)

    usleep(15000);    

    Xil_Out32(ISC0901_CTRL_REG_ADDR, (1<<SENSOR_CORE_PWR_ENA) | (1<<SENSOR_IO_PWR_ENA)); // enable VCC_IO (2.5V)

    usleep(3000); 

    Xil_Out32(ISC0901_CTRL_REG_ADDR, (1<<SENSOR_BIAS_VOLT_SEL) | (1<<SENSOR_BIAS_BOOST_PWR_ENA) | (1<<SENSOR_CORE_PWR_ENA) | (1<<SENSOR_IO_PWR_ENA)); // enable 11V and SENSOR_BIAS_BOOST_PWR_ENA
    usleep(1000);        
    Xil_Out32(ISC0901_CTRL_REG_ADDR, (1<<SENSOR_BIAS_VOLT_SEL) | (1<<SENSOR_BIAS_BOOST_PWR_ENA) | (1<<SENSOR_BIAS_PWR_ENA) | (1<<SENSOR_CORE_PWR_ENA) | (1<<SENSOR_IO_PWR_ENA) | (1<<SENSOR_RSTN) | (1<<ISC0901_RESET_N)); // enable VCC_BIAS (11.0V) / Cancel reset, activates clock for sensor in ISC0901_capture, enable ISC0901
}


void disable_ISC0901()
{
    Xil_Out32(ISC0901_CTRL_REG_ADDR, 0x00); // Reset    
}