#include "heatmap.h"
#include "xparameters.h"
#include <stdint.h>
#include <math.h>
#include "xil_io.h"

// a "heat" color map inspired by: http://www.andrewnoske.com/wiki/Code_-_heatmaps_and_color_gradients
typedef struct {
    uint8_t r, g, b;
} RGBColor;

#define RGB_BLACK {0, 0, 0} 
#define RGB_RED {255, 38, 0} 
#define RGB_YELLOW {255, 255, 0} 
#define RGB_GREEN {0, 249, 0} 
#define RGB_BLUE {4, 51, 255}
#define RGB_ORANGE {255, 147, 0}
#define RGB_CYAN {0, 253, 255}
#define RGB_MAGENTA {255, 64, 255}
#define RGB_PURPLE {148, 33, 146}
#define RGB_WHITE {255, 255, 255} 

const uint16_t HEAT_MAP_SIZE = 256;         // the number of colors in the heat map (must be <= 256)

const RGBColor colors[] = {RGB_BLACK, RGB_BLUE, RGB_CYAN, RGB_GREEN, RGB_YELLOW, RGB_RED, RGB_WHITE};
//const RGBColor colors[] = {RGB_BLUE, RGB_CYAN, RGB_GREEN, RGB_YELLOW, RGB_RED};
//const RGBColor colors[] = {RGB_BLUE, RGB_RED};
//const RGBColor colors[] = {RGB_BLACK, RGB_BLUE, RGB_MAGENTA, RGB_ORANGE, RGB_YELLOW, RGB_WHITE};
//const RGBColor colors[] = {RGB_BLACK, RGB_WHITE};

void init_heatmap()
{
    const int numColors = sizeof(colors) / sizeof(RGBColor);
    for (int c = 0; c < HEAT_MAP_SIZE; c++) {
        float value = c * (numColors-1) / (float)(HEAT_MAP_SIZE-1);
        int idx1 = (int)value;
        uint8_t red, green, blue;
        if (idx1 == value) {
            red = colors[idx1].r;
            green = colors[idx1].g;
            blue = colors[idx1].b;
        } else {
            int idx2 = idx1 + 1;
            float fractBetween = value - idx1;
            red = (int)(round((colors[idx2].r - colors[idx1].r) * fractBetween + colors[idx1].r));
            green = (int)(round((colors[idx2].g - colors[idx1].g) * fractBetween + colors[idx1].g));
            blue = (int)(round((colors[idx2].b - colors[idx1].b) * fractBetween + colors[idx1].b));
        }
        /*!
        @brief   Given 8-bit red, green and blue values, return a 'packed'
                16-bit color value in '565' RGB format (5 bits red, 6 bits
                green, 5 bits blue). This is just a mathematical operation,
                no hardware is touched.
        @param   red    8-bit red brightnesss (0 = off, 255 = max).
        @param   green  8-bit green brightnesss (0 = off, 255 = max).
        @param   blue   8-bit blue brightnesss (0 = off, 255 = max).
        @return  'Packed' 16-bit color value (565 format).
        */
        Xil_Out16(XPAR_AXI_BRAM_1_BASEADDRESS+2*c, ((red & 0xF8) << 8) | ((green & 0xFC) << 3) | (blue >> 3));
        //palette[c] = tft.color565(red, green, blue);
    }
}