#include "lcd.h"
#include "fonts.h"
#include "xil_io.h"
#include <stdlib.h>

uint16_t backcolor = YELLOW;    // color of background
uint16_t textcolor = BLACK;

void set_pixel(uint16_t pos_x, uint16_t pos_y, uint16_t color)
{
    Xil_Out16(LCD_BUFFER_ADDRESS + (pos_y*LCD_WIDTH+pos_x)*2, color);
}

void fill_rect(uint16_t start_x, uint16_t start_y, uint16_t end_x, uint16_t end_y, uint16_t color)
{
    uint16_t x, y;

    for(y=start_y; y < end_y; y++){
        for(x=start_x; x < end_x; x++){
                Xil_Out16(LCD_BUFFER_ADDRESS + (y*LCD_WIDTH+x)*2, color);       
        }        
    }
}

void lcd_drawLine(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t color){
//    if( x1 > DISPLAY_WIDTH-1 ||
//       x2 > DISPLAY_WIDTH-1 ||
//       y1 > DISPLAY_HEIGHT-1 ||
//       y2 > DISPLAY_HEIGHT-1) return;
    int dx =  abs(x2-x1), sx = x1<x2 ? 1 : -1;
    int dy = -abs(y2-y1), sy = y1<y2 ? 1 : -1;
    int err = dx+dy, e2; /* error value e_xy */
    
    while(1){
        set_pixel(x1, y1, color);
        if (x1==x2 && y1==y2) break;
        e2 = 2*err;
        if (e2 > dy) { err += dy; x1 += sx; } /* e_xy+e_x > 0 */
        if (e2 < dx) { err += dx; y1 += sy; } /* e_xy+e_y < 0 */
    }
}

void lcd_drawCircle(uint16_t center_x, uint16_t center_y, uint16_t radius, uint16_t color){
    if( ((center_x + radius) > LCD_WIDTH-1) ||
       ((center_y + radius) > LCD_HEIGHT-1) ||
       center_x < radius ||
       center_y < radius) return;
    int16_t f = 1 - radius;
    int16_t ddF_x = 1;
    int16_t ddF_y = -2 * radius;
    int16_t x = 0;
    int16_t y = radius;
    
    set_pixel(center_x  , center_y+radius, color);
    set_pixel(center_x  , center_y-radius, color);
    set_pixel(center_x+radius, center_y  , color);
    set_pixel(center_x-radius, center_y  , color);
    
    while (x<y) {
        if (f >= 0) {
            y--;
            ddF_y += 2;
            f += ddF_y;
        }
        x++;
        ddF_x += 2;
        f += ddF_x;
        
        set_pixel(center_x + x, center_y + y, color);
        set_pixel(center_x - x, center_y + y, color);
        set_pixel(center_x + x, center_y - y, color);
        set_pixel(center_x - x, center_y - y, color);
        set_pixel(center_x + y, center_y + x, color);
        set_pixel(center_x - y, center_y + x, color);
        set_pixel(center_x + y, center_y - x, color);
        set_pixel(center_x - y, center_y - x, color);
    }
}

void lcd_fillCircle(uint16_t center_x, uint16_t center_y, uint16_t radius, uint16_t color) {
    for(uint16_t i=0; i<= radius; i++){
        lcd_drawCircle(center_x, center_y, i, color);
    }
}

/*
 Writes a glyph("letter") to the display at location x,y
 (adapted function from the MJK-code)
 Arguments are:
    column    - x corrdinate of the left part of glyph
    row       - y coordinate of the top part of glyph
    width     - size in pixels of the width of the glyph
    height    - size in pixels of the height of the glyph
    glyph     - an unsigned char pointer to the glyph pixels
                to write assumed to be of length "width"
*/
void lcd_glyph(uint16_t x, uint16_t y, uint8_t width, uint8_t height, char *glyph_ptr, uint8_t store_width)
{
	uint8_t char_mask;
	uint16_t position_x=0, i;
	uint16_t counter_y;
	char *glyph_ptr_temp;

	for(counter_y = 0; counter_y < (height); counter_y++)	//Zeichne alle Reihen des Zeichens
	{
		glyph_ptr_temp = glyph_ptr;

		for (i=0, position_x=0; i<(1+width/8); i++, glyph_ptr_temp++)	//Zeichne alle Pixel in aktueller Reihe
		{
			char_mask=128;		//Maske, um die Pixel an entsprechender Stelle herauszufiltern

			while((char_mask > 0) && (position_x < width))	//alle 8 Pixel zeichnen, Abbruch bei Zeichenbreite
			{
				if((*glyph_ptr_temp) & char_mask)	//Wenn Pixel vorhanden
				{
					Xil_Out16(LCD_BUFFER_ADDRESS + ((y+counter_y)*LCD_WIDTH+x+position_x)*2, textcolor);	//setze Pixel
				}
				else
				{
					Xil_Out16(LCD_BUFFER_ADDRESS + ((y+counter_y)*LCD_WIDTH+x+position_x)*2, backcolor);	//lösche Pixel
				}
				position_x++;
				char_mask /= 2;
			}
		}

		glyph_ptr+=store_width;
	}
}

/*
 Prints the given string at location x,y in the specified font.
 Prints each character given via calls to lcd_glyph. The entry string
 is null terminated. (adapted function from the MJK-code)
 Arguments are:
	left       coordinate of left start of string.
	top        coordinate of top of string.
	font       font number to use for display (see fonts.h)
	str        text string to display (null-terminated)
*/
void lcd_text_intern(uint16_t left, uint16_t top, uint8_t font, char *string_ptr)
 {
   uint16_t x = left;
   char zeichen, *zeichen_ptr;
   uint8_t width;
   uint8_t height, defaultheight;
   uint8_t store_width;
   uint8_t *width_table_ptr;
   uint8_t *glyph_table_ptr;
   uint8_t glyph_beg, glyph_end;
   uint8_t fixedwidth;

   defaultheight = 	fonts[font].glyph_height;
   store_width = 	fonts[font].store_width;
   width_table_ptr = (uint8_t*) fonts[font].width_table;
   glyph_table_ptr = (uint8_t*) fonts[font].glyph_table;
   glyph_beg  = 	fonts[font].glyph_beg;
   glyph_end  = 	fonts[font].glyph_end;
   fixedwidth = 	fonts[font].fixed_width;


    zeichen = *string_ptr;

   do
   {
      // make zero based index into the font data arrays
      zeichen -= glyph_beg;
      if(fixedwidth == 0)
         width=*(width_table_ptr+zeichen);
      else
         width = fixedwidth;

      height = defaultheight;
      zeichen_ptr = (char*)(glyph_table_ptr + (zeichen * store_width * height)) ;

      lcd_glyph(x,top,width,height,zeichen_ptr,store_width);  // plug symbol into buffer

      x += width;		// move right for next character
      string_ptr++;		// point to next character in string

	zeichen = *string_ptr;

    }while(zeichen != '\0'); // while(*str != 0x00)
 }

//String aus dem RAM schreiben
void lcd_text(uint16_t left, uint16_t top, uint8_t font, char *string_ptr)
{
	lcd_text_intern(left, top, font, string_ptr);
}