/******************************************************************************
* Copyright (C) 2023 Advanced Micro Devices, Inc. All Rights Reserved.
* SPDX-License-Identifier: MIT
******************************************************************************/
/*
 * helloworld.c: simple test application
 *
 * This application configures UART 16550 to baud rate 9600.
 * PS7 UART (Zynq) is not initialized by this application, since
 * bootrom/bsp configures it to baud rate 115200
 *
 * ------------------------------------------------
 * | UART TYPE   BAUD RATE                        |
 * ------------------------------------------------
 *   uartns550   9600
 *   uartlite    Configurable only in HW design
 *   ps7_uart    115200 (configured by bootrom/bsp)
 */

#include <stdio.h>
#include "xparameters.h"
#include "platform.h"
#include "xil_printf.h"
#include "xil_types.h"
#include "xil_io.h"
#include "xil_testmem.h"

#include "sleep.h"
#include <inttypes.h>
#include <stdint.h>

#include "port.h"
#include "memory_config.h"
#include "vdma.h"
#include "fonts.h"
#include "lcd.h"
#include "vdma.h"
#include "heatmap.h"
#include "ISC0901.h"
#include "dip.h"
#include "ram_address.h"

int main()
{	
    int i, value = 0, x, y;
    
    init_platform();
    disable_ISC0901();

    sleep(1);

    init_ports();

    run_memory_test(); 

    print("Hello World\r");
    print("Successfully ran Hello World application\r");    

    for(i=0; i<320*240; i++)
    { 
        if(i<640)
            Xil_Out32(RAM_CH1_PING_ADDR+4*i, RED | (RED << 16));
        else
            Xil_Out32(RAM_CH1_PING_ADDR+4*i, WHITE | (WHITE << 16));
            
        Xil_Out32(RAM_CH1_PONG_ADDR+4*i, BLUE | (BLUE << 16));
    }

    for(y=0; y<240; y++){
        for(x=0; x<320*4; x++){
            if(x<=255*2)    
                Xil_Out8(RAM_CH0_PING_ADDR+(x+y*320), x/2);       
            else
                Xil_Out8(RAM_CH0_PING_ADDR+(x+y*320), 255);          
        }        
    }
    //Normally the following code will set every pixel when every pixel is 8-bit
    /*for(y=0; y<240; y++){
        for(x=0; x<320; x++){
            if(x<=255)    
                Xil_Out8(XPAR_OPENHBMC_0_BASEADDR+(x+y*320), x);       
            else
                Xil_Out8(XPAR_OPENHBMC_0_BASEADDR+(x+y*320), 255);          
        }        
    }*/

    value = Xil_In32(VDMA_STAT_REG_ADDR);     
    xil_printf("Reading from VDMA_STAT_REG_ADDR: 0x%x\r", value);

    init_heatmap();
    
    init_vdma();
    enable_vdma();
    LCD_LED_LEVEL(20);            

    value = Xil_In32(VDMA_STAT_REG_ADDR);     
    xil_printf("Reading from VDMA_STAT_REG_ADDR: 0x%x\r", value); 

    char string_name[20] = "Hello World!";
    lcd_text(11,10,FONT_NINE_DOT, string_name);
    fill_rect(70, 50, 120, 100, BLUE);
    lcd_drawLine(10,200,300,150, GREEN);
    lcd_fillCircle(200,200,10,BLUE);

    init_dip(); 
    init_ISC0901();
    
    i = 0;
    
    while(1)
    {
        usleep(500000);

        LED_GREEN_ON();
        //print("LED on\r");

        usleep(500000);

        //Xil_Out32(VDMA_CH1_CONFIG_ADDR, 0x0f | (0<<30) | (1<<31)); //Set Lastpix and interlaced mode 

        LED_GREEN_OFF();
        //print("LED off\r");  

        usleep(500000);

        LED_RED_ON();
        //print("LED on\r");

        usleep(500000);

        LED_RED_OFF();
        //print("LED off\r");

        //Xil_Out32(VDMA_CH1_CONFIG_ADDR, 0x0f | (0<<30) | (0<<31)); //Set Lastpix and interlaced mode 

        

        if(i++<5)
        {
            xil_printf("Reading from VDMA_STAT_REG_ADDR: 0x%x\r", value); 
            
        /*              DMA_XFER_OKAY   = 4'b1000, 0x8
                        DMA_XFER_SLVERR = 4'b0100, 0x4
                        DMA_XFER_DECERR = 4'b0010, 0x2
                        DMA_XFER_INTERR = 4'b0001; 0x1
                        
                        ST_GET_MM2S_XFER_STAT   = 4'b1011, 0xB
                        ST_DATMOV_MM2S_XFER     = 4'b1010, 0xA
                        ST_DATMOV_MM2S_CMD_1    = 4'b1001, 0x9
                        ST_ERROR                = 4'd0,
                        ST_RESET                = 4'd1,
                        ST_HANDLE_CH0           = 4'd2,
                        ST_HANDLE_CH1           = 4'd3,
                        ST_HANDLE_CH2           = 4'd4,
                        ST_HANDLE_CH3           = 4'd5,
                        ST_HANDLE_CH4           = 4'd6,
                        ST_HANDLE_CH5           = 4'd7,*/  

            xil_printf("Reading from DIP_CTRL_REG_ADDR: 0x%x\r", read_dip_stat_reg());          
        }
    }

    cleanup_platform();
    return 0;
}
